/* Declarations of functions and data types used for SHA1 sum library
 * functions.
 * part of Cumulus: Efficient Filesystem Backup to the Cloud
 *
 * Copyright (C) 2000, 2001, 2003, 2005 Free Software Foundation, Inc.
 * Copyright (C) 2006-2007  The Regents of the University of California
 *
 * Original code (in C) is taken from GNU coreutils (Debian package 5.97-5).
 * Modifications by Michael Vrable <mvrable@cs.ucsd.edu> to integrate into
 * Cumulus.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 */

#ifndef SHA1_H
# define SHA1_H 1

# include <stdio.h>
# include <stdint.h>

#include <string>

typedef uint32_t md5_uint32;

/* Structure to save state of computation between the single steps.  */
struct sha1_ctx
{
  md5_uint32 A;
  md5_uint32 B;
  md5_uint32 C;
  md5_uint32 D;
  md5_uint32 E;

  md5_uint32 total[2];
  md5_uint32 buflen;
  char buffer[128] __attribute__ ((__aligned__ (__alignof__ (md5_uint32))));
};


/* Initialize structure containing state of computation. */
extern void sha1_init_ctx (struct sha1_ctx *ctx);

/* Starting with the result of former calls of this function (or the
   initialization function update the context for the next LEN bytes
   starting at BUFFER.
   It is necessary that LEN is a multiple of 64!!! */
extern void sha1_process_block (const void *buffer, size_t len,
				struct sha1_ctx *ctx);

/* Starting with the result of former calls of this function (or the
   initialization function update the context for the next LEN bytes
   starting at BUFFER.
   It is NOT required that LEN is a multiple of 64.  */
extern void sha1_process_bytes (const void *buffer, size_t len,
				struct sha1_ctx *ctx);

/* Process the remaining bytes in the buffer and put result from CTX
   in first 20 bytes following RESBUF.  The result is always in little
   endian byte order, so that a byte-wise output yields to the wanted
   ASCII representation of the message digest.

   IMPORTANT: On some systems it is required that RESBUF be correctly
   aligned for a 32 bits value.  */
extern void *sha1_finish_ctx (struct sha1_ctx *ctx, void *resbuf);


/* Put result from CTX in first 20 bytes following RESBUF.  The result is
   always in little endian byte order, so that a byte-wise output yields
   to the wanted ASCII representation of the message digest.

   IMPORTANT: On some systems it is required that RESBUF is correctly
   aligned for a 32 bits value.  */
extern void *sha1_read_ctx (const struct sha1_ctx *ctx, void *resbuf);

/* An object-oriented wrapper around checksumming functionality. */
class SHA1Checksum {
private:
    struct sha1_ctx ctx;
    char resbuf[20] __attribute__ ((__aligned__ (__alignof__ (md5_uint32))));

public:
    SHA1Checksum();
    ~SHA1Checksum();

    void process(const void *data, size_t len);
    bool process_file(const char *filename);
    const uint8_t *checksum();
    size_t checksum_size() const { return 20; }
    std::string checksum_str();
};

#endif
